// ******************************************************************************
// Copyright (c) 2007 srcX Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// ******************************************************************************

#include "csrcxplugin.h"
#include "dlls/iplayerinfo.h"
#include "eiface.h"
#include "filesystem.h"
#include "engine/IEngineTrace.h"
#include "vstdlib/random.h"
#include "icvar.h"

// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

extern IPlayerInfoManager* g_pPlayerInfoManager;
extern IBotManager* g_pBotManager;
extern IVEngineServer* g_pEngine;
extern IGameEventManager2* g_pGameEventManager;
extern IFileSystem* g_pFileSystem; 
extern IServerPluginHelpers* g_pHelpers;
extern IEngineTrace* g_pEngineTrace;
extern IUniformRandomStream* g_pRandomStr;
extern ICvar* g_pConVars;
extern CGlobalVars* g_pGlobals;

// Function to initialize any ConVars/ConCommands we have defined.
void InitConVars();

extern ConVar gameDirConVar;

CsrcXPlugin::CsrcXPlugin():
	_nClientCommandIndex(0)
{
}

// ******************************************************************************
// Initialize the plugin to run.
// Return false if there is an error during startup.
// ******************************************************************************
bool CsrcXPlugin::Load(CreateInterfaceFn interfaceFactory, CreateInterfaceFn gameServerFactory)
{
	if (!(g_pPlayerInfoManager = (IPlayerInfoManager*)gameServerFactory(INTERFACEVERSION_PLAYERINFOMANAGER, NULL)) ||
		!(g_pBotManager = (IBotManager*)gameServerFactory(INTERFACEVERSION_PLAYERBOTMANAGER, NULL)) ||
		!(g_pEngine = (IVEngineServer*)interfaceFactory(INTERFACEVERSION_VENGINESERVER, NULL)) ||
		!(g_pGameEventManager = (IGameEventManager2*)interfaceFactory(INTERFACEVERSION_GAMEEVENTSMANAGER2, NULL)) ||
		!(g_pFileSystem = (IFileSystem*)interfaceFactory(FILESYSTEM_INTERFACE_VERSION, NULL)) ||
		!(g_pHelpers = (IServerPluginHelpers*)interfaceFactory(INTERFACEVERSION_ISERVERPLUGINHELPERS, NULL)) || 
		!(g_pEngineTrace = (IEngineTrace*)interfaceFactory(INTERFACEVERSION_ENGINETRACE_SERVER, NULL)) ||
		!(g_pRandomStr = (IUniformRandomStream*)interfaceFactory(VENGINE_SERVER_RANDOM_INTERFACE_VERSION, NULL)) ||
		!(g_pConVars = (ICvar*)interfaceFactory(VENGINE_CVAR_INTERFACE_VERSION, NULL)))
	{
		// We cannot function without these interfaces.
		return false;
	}

	g_pGlobals = g_pPlayerInfoManager->GetGlobalVars();

	// Registers any variables or commands we have defined.
	InitConVars();
	MathLib_Init(2.2f, 2.2f, 0.0f, 2.0f);

	// Get the game directory.
	char buffer[1024];
	g_pEngine->GetGameDir(buffer, sizeof(buffer));
	gameDirConVar.SetValue(buffer);

	return true;
}

// ******************************************************************************
// Called when the plugin should be shutdown.
// ******************************************************************************
void CsrcXPlugin::Unload()
{
}

// ******************************************************************************
// Called when a plugins execution is stopped but the plugin is not unloaded.
// ******************************************************************************
void CsrcXPlugin::Pause()
{
}

// ******************************************************************************
// Called when a plugin should start executing again (sometime after a Pause() call).
// ******************************************************************************
void CsrcXPlugin::UnPause()
{
}

// ******************************************************************************
// Returns string describing current plugin. e.g., Admin-Mod.
// ******************************************************************************
const char* CsrcXPlugin::GetPluginDescription()
{
	return "srcX Plugin, v" SRCX_VERSION ", srcX Team";
}

// ******************************************************************************
// Called any time a new level is started (after GameInit() also on level transitions within a game).
// ******************************************************************************
void CsrcXPlugin::LevelInit(char const* pszMapName)
{
}

// ******************************************************************************
// The server is about to activate.
// ******************************************************************************
void CsrcXPlugin::ServerActivate(edict_t* pEdictList, int nEdictCount, int nClientMax)
{
}

// ******************************************************************************
// The server should run physics/think on all edicts.
// ******************************************************************************
void CsrcXPlugin::GameFrame(bool bSimulating)
{
}

// ******************************************************************************
// Called when a level is shutdown (including changing levels).
// ******************************************************************************
void CsrcXPlugin::LevelShutdown()
{
}

// ******************************************************************************
// Client is going active.
// ******************************************************************************
void CsrcXPlugin::ClientActive(edict_t* pEdict)
{
}

// ******************************************************************************
// Client is disconnecting from server.
// ******************************************************************************
void CsrcXPlugin::ClientDisconnect(edict_t* pEdict)
{
}

// ******************************************************************************
// Client is connected and should be put in the game.
// ******************************************************************************
void CsrcXPlugin::ClientPutInServer(edict_t* pEdict, char const* pszClientName)
{
}

// ******************************************************************************
// Sets the client index for the client who typed the command into their console.
// ******************************************************************************
void CsrcXPlugin::SetCommandClient(int nIndex)
{
	_nClientCommandIndex = nIndex;
}

// ******************************************************************************
// Client changed one/several replicated cvars (name etc).
// ******************************************************************************
void CsrcXPlugin::ClientSettingsChanged(edict_t* pEdict)
{
}

// ******************************************************************************
// Client is connecting to server (set pbAllowConnect to false to reject the connection).
//   You can specify a rejection message by writing it into pszReject.
// ******************************************************************************
PLUGIN_RESULT CsrcXPlugin::ClientConnect(bool* pbAllowConnect, edict_t* pEdict, const char* pszClientName, const char* pszAddress, char* pszReject, int MaxRejectLen)
{
	return PLUGIN_CONTINUE;
}

// ******************************************************************************
// Client has typed a command at the console.
// ******************************************************************************
PLUGIN_RESULT CsrcXPlugin::ClientCommand(edict_t* pEdict)
{
	return PLUGIN_CONTINUE;
}

// ******************************************************************************
// Client has had their network id setup and validated.
// ******************************************************************************
PLUGIN_RESULT CsrcXPlugin::NetworkIDValidated(const char* pszClientName, const char* pszNetworkID)
{
	return PLUGIN_CONTINUE;
}

// ******************************************************************************
// FireGameEvent is called by g_pGameEventManager if an event just occured.
// pGameEvent memory will be freed by manager if not needed anymore.
// ******************************************************************************
void CsrcXPlugin::FireGameEvent(IGameEvent* pGameEvent)
{
}
