// ******************************************************************************
// Copyright (c) 2007 srcX Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// ******************************************************************************

#include "csrcxplugin.h"
#include "dlls/iplayerinfo.h"
#include "eiface.h"
#include "filesystem.h"
#include "engine/IEngineTrace.h"
#include "vstdlib/random.h"
#include "icvar.h"
#include "edict.h"

// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

IPlayerInfoManager* g_pPlayerInfoManager = NULL;
IBotManager* g_pBotManager = NULL;
IVEngineServer* g_pEngine = NULL;
IGameEventManager2* g_pGameEventManager = NULL;
IFileSystem* g_pFileSystem = NULL;
IServerPluginHelpers* g_pHelpers = NULL;
IEngineTrace* g_pEngineTrace = NULL;
IUniformRandomStream* g_pRandomStr = NULL;
ICvar* g_pConVars = NULL;
CGlobalVars* g_pGlobals = NULL;

CsrcXPlugin g_srcXPlugin;
EXPOSE_SINGLE_INTERFACE_GLOBALVAR(CsrcXPlugin, IServerPluginCallbacks, INTERFACEVERSION_ISERVERPLUGINCALLBACKS, g_srcXPlugin);

// ******************************************************************************
// Function to convert a userid to an edict_t* type.
// ******************************************************************************
edict_t* UTIL_UserIDToEdict(int userid)
{
	if (userid == -1)
		return NULL;

	// Look for the user.
	for (int i = 1; i <= g_pGlobals->maxClients; ++i)
	{
		edict_t* p_client = g_pEngine->PEntityOfEntIndex(i);
		if (!p_client || p_client->IsFree())
			continue;

		if (g_pEngine->GetPlayerUserId(p_client) == userid)
			return p_client;
	}

	return NULL;
}

// ******************************************************************************
// Function to convert an entity index to an edict_t* type.
// ******************************************************************************
edict_t* UTIL_IndexToEdict(int index)
{
	// Look for the client.
	for (int i = 0; i < g_pGlobals->maxEntities; ++i)
	{
		edict_t* p_edict = g_pEngine->PEntityOfEntIndex(i);
		if (!p_edict || p_edict->IsFree())
			continue;

		if (i == index)
			return p_edict;
	}

	return NULL;
}

// ******************************************************************************
// Defines some ConVars for srcX.
// ******************************************************************************
ConVar versionConVar("srcx_version", SRCX_VERSION, FCVAR_NOTIFY | FCVAR_REPLICATED, "Holds the version of the srcX plugin.");
ConVar gameDirConVar("srcx_gamedir", "", 0, "Holds the path to the game directory.");
