// ******************************************************************************
// Copyright (c) 2007 srcX Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// ******************************************************************************

#include "icvar.h"
#include "eiface.h"

// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

extern IVEngineServer* g_pEngine;
extern ICvar* g_pConVars;

CON_COMMAND(sx_bitwise, "Performs bitwise mathematical functions.")
{
	const int nArgc = g_pEngine->Cmd_Argc();
	if (nArgc < 3)
	{
		Msg("[srcX] Parameters: sx_bitwise <var> <operator> [value]\n");
		return;
	}

	// Did we provide a valid variable?
	const char* p_szArg1 = g_pEngine->Cmd_Argv(1);
	ConVar* p_returnConVar = g_pConVars->FindVar(p_szArg1);
	if (!p_returnConVar)
	{
		Msg("[srcX] Error: Could not find variable \"%s\".\n", p_szArg1);
		return;
	}

	// Did we provide a valid operation?
	const char* p_szArg2 = g_pEngine->Cmd_Argv(2);
	if (!Q_stricmp(p_szArg2, "or") || !Q_strcmp(p_szArg2, "|"))
	{
		const char* p_szArg2 = g_pEngine->Cmd_Argv(2);
		const unsigned int nArg2 = Q_atoi(p_szArg2);

		// Do the operation.
		const unsigned int nResult = p_returnConVar->GetInt() | nArg2;
		char buffer[16];
		Q_snprintf(buffer, sizeof(buffer), "%u", nResult);
		p_returnConVar->SetValue(buffer);
	}
	else if (!Q_stricmp(p_szArg2, "and") || !Q_strcmp(p_szArg2, "&"))
	{
		const char* p_szArg2 = g_pEngine->Cmd_Argv(2);
		const unsigned int nArg2 = Q_atoi(p_szArg2);

		// Do the operation.
		const unsigned int nResult = p_returnConVar->GetInt() & nArg2;
		char buffer[16];
		Q_snprintf(buffer, sizeof(buffer), "%u", nResult);
		p_returnConVar->SetValue(buffer);
	}
	else if (!Q_stricmp(p_szArg2, "xor") || !Q_strcmp(p_szArg2, "^"))
	{
		const char* p_szArg2 = g_pEngine->Cmd_Argv(2);
		const unsigned int nArg2 = Q_atoi(p_szArg2);

		// Do the operation.
		const unsigned int nResult = p_returnConVar->GetInt() ^ nArg2;
		char buffer[16];
		Q_snprintf(buffer, sizeof(buffer), "%u", nResult);
		p_returnConVar->SetValue(buffer);
	}
	else if (!Q_stricmp(p_szArg2, "not") || !Q_strcmp(p_szArg2, "~"))
	{
		const char* p_szArg2 = g_pEngine->Cmd_Argv(2);
		const unsigned int nArg2 = Q_atoi(p_szArg2);

		// Do the operation.
		const unsigned int nResult = ~p_returnConVar->GetInt();
		char buffer[16];
		Q_snprintf(buffer, sizeof(buffer), "%u", nResult);
		p_returnConVar->SetValue(buffer);
	}
	else if (!Q_stricmp(p_szArg2, "lshift") || !Q_strcmp(p_szArg2, "<<"))
	{
		const char* p_szArg2 = g_pEngine->Cmd_Argv(2);
		const unsigned int nArg2 = Q_atoi(p_szArg2);

		// Do the operation.
		const unsigned int nResult = p_returnConVar->GetInt() << nArg2;
		char buffer[16];
		Q_snprintf(buffer, sizeof(buffer), "%u", nResult);
		p_returnConVar->SetValue(buffer);
	}
	else if (!Q_stricmp(p_szArg2, "rshift") || !Q_strcmp(p_szArg2, ">>"))
	{
		const char* p_szArg2 = g_pEngine->Cmd_Argv(2);
		const unsigned int nArg2 = Q_atoi(p_szArg2);

		// Do the operation.
		const unsigned int nResult = p_returnConVar->GetInt() >> nArg2;
		char buffer[16];
		Q_snprintf(buffer, sizeof(buffer), "%u", nResult);
		p_returnConVar->SetValue(buffer);
	}
	else
	{
		Msg("[srcX] Error: \"%s\" is a bad operation.\n", p_szArg2);
	}
}
