// ******************************************************************************
// Copyright (c) 2007 srcX Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// ******************************************************************************

#include "convar.h"
#include "eiface.h"
#include "csrcxplugin.h"
#include "edict.h"

// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

extern IVEngineServer* g_pEngine;
extern IServerPluginHelpers* g_pHelpers;

extern CsrcXPlugin g_srcXPlugin;

edict_t* UTIL_UserIDToEdict(int userid);

CON_COMMAND(sx_escmenu, "Sends an ESC menu to a client.")
{
	const int argc = g_pEngine->Cmd_Argc();
	if ((argc < 6) || (argc > 13))
	{
		Msg("[srcX] Parameters: sx_escmenu <time> <userid> <title> <text> <button1> [button...] [button8]\n");
		return;
	}

	// Did we provide a valid time?
	const char* p_arg1 = g_pEngine->Cmd_Argv(1);
	const int time = Q_atoi(p_arg1);
	if ((time < 10) || (time > 200))
	{
		Msg("[srcX] Error: \"%s\" is an invalid specifier for <time>.\n[srcX] Error: <time> must be at least 10 or at the most 200.\n", p_arg1);
		return;
	}

	// Did we provide a valid userid?
	const char* p_arg2 = g_pEngine->Cmd_Argv(2);
	const int userid = Q_atoi(p_arg2);
	edict_t* p_target = UTIL_UserIDToEdict(userid);
	if (!p_target)
	{
		Msg("[srcX] Error: \"%s\" is an invalid userid.\n", p_arg2);
		return;
	}

	// Look for new lines.
	char* p_arg4 = g_pEngine->Cmd_Argv(4);
	for (int i = 0; p_arg4[i] != '\0'; i++)
	{
		if ((p_arg4[i] == '\\') && (p_arg4[i + 1] == 'n'))
		{
			p_arg4[i] = '\n';
			for (int j = i + 1; p_arg4[j] != '\0'; j++)
				p_arg4[j] = p_arg4[j + 1];
		}
	}

	// Create the menu.
	KeyValues* p_menu = new KeyValues("menu");
	p_menu->SetString("title", g_pEngine->Cmd_Argv(3));
	p_menu->SetInt("level", 0);
	p_menu->SetColor("color", Color(255, 255, 255, 255));
	p_menu->SetInt("time", time);
	p_menu->SetString("msg", p_arg4);

	// Add the buttons.
	for (int i = 1, argi = 5; i <= 8; i++, argi++)
	{
		if (argi > (argc - 1))
			break;

		char key[2], cmd[17];

		Q_snprintf(key, sizeof(key), "%i", i);
		Q_snprintf(cmd, sizeof(cmd), "menuselect %i;//\"", i);

		// Add the button.
		KeyValues* p_button = p_menu->FindKey(key, true);
		p_button->SetString("msg", g_pEngine->Cmd_Argv(argi));
		p_button->SetString("command", cmd);
	}

	g_pHelpers->CreateMessage(p_target, DIALOG_MENU, p_menu, &g_srcXPlugin);
	p_menu->deleteThis();
}
