// ******************************************************************************
// Copyright (c) 2007 srcX Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// ******************************************************************************

#include "convar.h"
#include "eiface.h"
#include "csrcxplugin.h"
#include "edict.h"

// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

extern IVEngineServer* g_pEngine;
extern IServerPluginHelpers* g_pHelpers;

extern CsrcXPlugin g_srcXPlugin;

edict_t* UTIL_UserIDToEdict(int userid);

CON_COMMAND(sx_esctextbox, "Sends an ESC text-box to a client.")
{
	if (g_pEngine->Cmd_Argc() != 5)
	{
		Msg("[srcX] Parameters: sx_esctextbox <time> <userid> <title> <text>\n");
		return;
	}

	// Did we provide a valid time?
	const char* p_arg1 = g_pEngine->Cmd_Argv(1);
	const int time = Q_atoi(p_arg1);
	if ((time < 10) || (time > 200))
	{
		Msg("[srcX] Error: \"%s\" is an invalid specifier for <time>.\n[srcX] Error: <time> must be at least 10 or at the most 200.\n", p_arg1);
		return;
	}

	// Did we provide a valid userid?
	const char* p_arg2 = g_pEngine->Cmd_Argv(2);
	const int userid = Q_atoi(p_arg2);
	edict_t* p_target = UTIL_UserIDToEdict(userid);
	if (!p_target)
	{
		Msg("[srcX] Error: \"%s\" is an invalid userid.\n", p_arg2);
		return;
	}

	// Look for new lines and tab spaces.
	char* p_arg4 = g_pEngine->Cmd_Argv(4);
	for (int i = 0; p_arg4[i] != '\0'; i++)
	{
		if ((p_arg4[i] == '\\') && (p_arg4[i + 1] == 'n'))
		{
			p_arg4[i] = '\n';
			for (int j = i + 1; p_arg4[j] != '\0'; j++)
				p_arg4[j] = p_arg4[j + 1];
		}
	}

	// Create the text-box.
	KeyValues* p_textBox = new KeyValues("textBox");
	p_textBox->SetString("title", g_pEngine->Cmd_Argv(3));
	p_textBox->SetInt("level", 0);
	p_textBox->SetColor("color", Color(255, 255, 255, 255));
	p_textBox->SetInt("time", time);
	p_textBox->SetString("msg", p_arg4);

	g_pHelpers->CreateMessage(p_target, DIALOG_TEXT, p_textBox, &g_srcXPlugin);
	p_textBox->deleteThis();
}
