// ******************************************************************************
// Copyright (c) 2007 srcX Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// ******************************************************************************

#include "icvar.h"
#include "eiface.h"

// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

extern IVEngineServer* g_pEngine;
extern ICvar* g_pConVars;

edict_t* UTIL_IndexToEdict(int index);

CON_COMMAND(sx_getentmodelindex, "Gets the model index of an entity and stores it into a variable.")
{
	if (g_pEngine->Cmd_Argc() != 3)
	{
		Msg("[srcX] Parameters: sx_getentmodelindex <var> <index>\n");
		return;
	}

	// Did we provide a valid variable?
	const char* p_arg1 = g_pEngine->Cmd_Argv(1);
	ConVar* p_returnConVar = g_pConVars->FindVar(p_arg1);
	if (!p_returnConVar)
	{
		Msg("[srcX] Error: Could not find variable \"%s\".\n", p_arg1);
		return;
	}

	// Did we provide a valid index?
	const char* p_arg2 = g_pEngine->Cmd_Argv(2);
	const int index = Q_atoi(p_arg2);
	edict_t* p_target = UTIL_IndexToEdict(index);
	if (!p_target)
	{
		Msg("[srcX] Error: \"%s\" is an invalid index.\n", p_arg2);
		return;
	}

	// Get the model index and store it into <var>.
	const IServerEntity* const p_servEntity = p_target->GetIServerEntity();
	p_returnConVar->SetValue(p_servEntity->GetModelIndex());
}
